package com.sztzjy.util.pdfUtils;

import com.itextpdf.text.*;
import com.itextpdf.text.Font;
import com.itextpdf.text.Image;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.*;
import lombok.SneakyThrows;
import org.springframework.stereotype.Component;

import java.awt.*;

@Component
public class HeaderAndFooterEvent extends PdfPageEventHelper {
    public static PdfTemplate tpl = null;
    public static boolean PAGE_NUMBER = false;
    public static Rectangle rect = PageSize.A4;

    private static Font font;

    public static String HeaderFooterFontName = "黑体";
    public static int HeaderFooterFontSize = 10;
    public static Color HeaderFooterFontColor = Color.BLACK;

    public static String HeaderLeft;
    public static String HeaderRight;
    public static String FooterLeft;
    public static String FooterRight;

    public static void SetPageSize(String type) {
        switch (type.trim()) {
            case "A4":
                rect = PageSize.A4;
                break;
            case "A8":
                rect = PageSize.A8;
                break;
        }
    }

    @SneakyThrows
    public static void SetFont(BaseColor color, String fontName, float size, int style) {
        BaseFont baseFont;
//        String FONT_PATH="C:/Windows/Fonts/";
        String FONT_PATH="/usr/share/fonts/";
        switch (fontName) {
            case "黑体":
//                baseFont = BaseFont.createFont("SIMHEI.TTF", BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
                baseFont = BaseFont.createFont(FONT_PATH+"simhei.ttf", BaseFont.IDENTITY_H, BaseFont.NOT_EMBEDDED);
                break;
            case "华文中宋":
//                baseFont = BaseFont.createFont("STZHONGS.TTF", BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
                baseFont = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
                break;
            case "宋体":
//                baseFont = BaseFont.createFont("SIMYOU.TTF", BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
                baseFont = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
                break;
            default:
//                baseFont = BaseFont.createFont("SIMYOU.TTF", BaseFont.IDENTITY_H, BaseFont.EMBEDDED);
                baseFont = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H", BaseFont.NOT_EMBEDDED);
                break;
        }
        font = new Font(baseFont, size, style,color);
    }

    @Override
    public void onEndPage(PdfWriter writer, Document document) {
        if (PAGE_NUMBER) {
            Font HeaderFooterFont = new Font(font.getBaseFont(), HeaderFooterFontSize, Font.NORMAL);
            Phrase header_left = new Phrase(HeaderLeft, HeaderFooterFont);
            Phrase header_right = new Phrase(HeaderRight, HeaderFooterFont);

            Phrase footer_left = new Phrase(FooterLeft, HeaderFooterFont);
            Phrase footer_center = new Phrase("第" + (writer.getPageNumber()) + "页/共 页", HeaderFooterFont);
            Phrase footer_right = new Phrase(FooterRight, HeaderFooterFont);

            PdfContentByte cb = writer.getDirectContent();
            cb.addTemplate(tpl, document.right() - 290 + document.leftMargin(), document.bottom() - 15);

            ColumnText.showTextAligned(cb, Element.ALIGN_CENTER, header_right,
                    document.right() - 50 + document.leftMargin(), document.top() + 15, 0);

            ColumnText.showTextAligned(cb, Element.ALIGN_CENTER, header_left,
                    document.right() - 500 + document.leftMargin(), document.top() + 15, 0);

            ColumnText.showTextAligned(cb, Element.ALIGN_CENTER, footer_left,
                    document.right() - 535 + document.leftMargin(), document.bottom() - 15, 0);

            ColumnText.showTextAligned(cb, Element.ALIGN_CENTER, footer_center,
                    document.right() - 300 + document.leftMargin(), document.bottom() - 15, 0);

            ColumnText.showTextAligned(cb, Element.ALIGN_CENTER, footer_right,
                    document.right() - 80 + document.leftMargin(), document.bottom() - 15, 0);

        }
    }

    @Override
    public void onStartPage(PdfWriter writer, Document document) {
        if (PAGE_NUMBER) {
            writer.setPageCount(writer.getPageNumber());
        }
    }

    @Override
    public void onCloseDocument(PdfWriter writer, Document document) {
        tpl.beginText();
        tpl.setFontAndSize(font.getBaseFont(), HeaderFooterFontSize);
        tpl.showText(String.valueOf(writer.getPageNumber()));
        tpl.endText();
        tpl.closePath();
    }

    public static Paragraph AddParagraph(String content, int Alignment, float SpacingAfter, float SpacingBefore, float MultipliedLeading) {
        Paragraph pra = new Paragraph(content, font);
        pra.setAlignment(Alignment);
        pra.setSpacingAfter(SpacingAfter);
        pra.setSpacingBefore(SpacingBefore);
        // pra.setMultipliedLeading(MultipliedLeading);
        return pra;
    }

    public static Paragraph AddParagraph(String content, int Alignment, float MultipliedLeading) {
        Paragraph pra = new Paragraph(content, font);
        pra.setAlignment(Alignment);
        // pra.setMultipliedLeading(MultipliedLeading);
        return pra;
    }

    public static Paragraph AddParagraph(String content, int Alignment) {
        Paragraph pra = new Paragraph(content, font);
        pra.setAlignment(Alignment);
        return pra;
    }

    public static Paragraph AddParagraph(String content) {
        Paragraph pra = new Paragraph(content, font);
        return pra;
    }

    public static PdfPCell AddCell(String content, int horizontalAlignment, int verticalAlignment, float borderWidth) {
        PdfPCell cell = new PdfPCell(new Phrase(content, font));
        cell.setHorizontalAlignment(horizontalAlignment);
        cell.setVerticalAlignment(verticalAlignment);
        cell.setBorderWidth(borderWidth);
        return cell;
    }

    public static PdfPCell AddCell(String content, int horizontalAlignment, int verticalAlignment) {
        PdfPCell cell = new PdfPCell(new Phrase(content, font));
        cell.setHorizontalAlignment(horizontalAlignment);
        cell.setVerticalAlignment(verticalAlignment);
        return cell;
    }

    public static PdfPCell AddCell(String content, int horizontalAlignment) {
        PdfPCell cell = new PdfPCell(new Phrase(content, font));
        cell.setHorizontalAlignment(horizontalAlignment);
        return cell;
    }

    public static PdfPCell AddCell(String content) {
        PdfPCell cell = new PdfPCell(new Phrase(content, font));
        return cell;
    }

    public static PdfPCell AddCell(Image image, int horizontalAlignment, int verticalAlignment, float borderWidth) {
        PdfPCell cell = new PdfPCell(image);
        cell.setHorizontalAlignment(horizontalAlignment);
        cell.setVerticalAlignment(verticalAlignment);
        cell.setBorderWidth(borderWidth);
        return cell;
    }

    public static PdfPCell AddCell(Image image, int horizontalAlignment, int verticalAlignment) {
        PdfPCell cell = new PdfPCell(image);
        cell.setHorizontalAlignment(horizontalAlignment);
        cell.setVerticalAlignment(verticalAlignment);
        return cell;
    }

    public static PdfPCell AddCell(Image image, int horizontalAlignment) {
        PdfPCell cell = new PdfPCell(image);
        cell.setHorizontalAlignment(horizontalAlignment);
        return cell;
    }

    public static PdfPCell AddCell(Image image) {
        PdfPCell cell = new PdfPCell(image);
        return cell;
    }

}
